package com.reproduce.error;

import java.math.BigInteger;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.persistence.EntityExistsException;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityNotFoundException;
import javax.persistence.Persistence;

public class ReproduceTest {

    public static void main(String[] args) {
        // initialise
        ReproduceTest test = new ReproduceTest();
        TimePeriod tp = new TimePeriod();
        tp.setPeriodType("Test");
        tp.setBeginTime(new GregorianCalendar(1965, 06, 13));
        tp.setEndTime(new GregorianCalendar(2011, 06, 13));
        // perform the test
        try {
            test.performTest(BigInteger.ONE, BigInteger.TEN, tp);
        } catch (Exception ex) {
            Logger.getLogger(ReproduceTest.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
    private EntityManagerFactory emf;

    public ReproduceTest() {
        emf = Persistence.createEntityManagerFactory("transformPU");
    }

    public EntityManager getEntityManager() {
        return emf.createEntityManager();
    }

    public void performTest(BigInteger x, BigInteger y, TimePeriod identifier)
            throws Exception {
        // obtain the moment
        Calendar now = Calendar.getInstance();
        MomentEntity moment = findMomentEntity(now.getTimeInMillis());
        if (moment == null) {
            create(new MomentEntity(now.getTimeInMillis()));
            moment = findMomentEntity(now.getTimeInMillis());
        }
        // create an instance of point on the chart
        PointEntity latestPoint = new PointEntity(moment);
        latestPoint.setX(x);
        latestPoint.setY(y);
        // create and update a Chart
        ChartEntity chart = new ChartEntity(identifier);
        create(chart);
        chart.addPoint(latestPoint);
        edit(chart);
    }

    public MomentEntity findMomentEntity(Long id) {
        EntityManager em = getEntityManager();
        try {
            return em.find(MomentEntity.class, id);
        } finally {
            em.close();
        }
    }

    public void create(MomentEntity moment) throws Exception {
        EntityManager em = null;
        try {
            em = getEntityManager();
            em.getTransaction().begin();
            em.persist(moment);
            em.getTransaction().commit();
        } catch (Exception ex) {
            if (findMomentEntity(moment.getDateTime()) != null) {
                throw new EntityExistsException("MomentEntity " + moment + " already exists.", ex);
            }
            throw ex;
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }

    public void create(ChartEntity chart) throws Exception  {
        EntityManager em = null;
        try {
            em = getEntityManager();
            em.getTransaction().begin();
            em.persist(chart);
            em.getTransaction().commit();
        } catch (Exception ex) {
            if (findChart(chart.getId()) != null) {
                throw new EntityExistsException("ChartEntity " + chart + " already exists.", ex);
            }
            throw ex;
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }

    public void edit(ChartEntity chart) throws Exception  {
        EntityManager em = null;
        try {
            em = getEntityManager();
            em.getTransaction().begin();
            chart = em.merge(chart);
            em.getTransaction().commit();
        } catch (Exception ex) {
            String msg = ex.getLocalizedMessage();
            if (msg == null || msg.length() == 0) {
                TimePeriod id = chart.getId();
                if (findChart(id) == null) {
                    throw new EntityNotFoundException("The ChartEntity with id " + id + " no longer exists.");
                }
            }
            throw  ex;
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }

    public ChartEntity findChart(TimePeriod id) {
        EntityManager em = getEntityManager();
        try {
            return em.find(ChartEntity.class, id);
        } finally {
            em.close();
        }
    }
}
